using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace SendFileTo
{
    class MAPI
    {
        const int MAPI_LOGON_UI = 0x00000001;
        const int MAPI_DIALOG = 0x00000008;
        const int maxAttachments = 200;

        readonly string[] errors;
        int m_lastError;

        List<MapiRecipDesc> m_recipients;
        List<string> m_attachments;

        enum HowTo
        { 
            MAPI_ORIG = 0, 
            MAPI_TO, 
            MAPI_CC,  //not supported
            MAPI_BCC  //not supported 
        };

        public MAPI()
        {
            errors = new string[] {
		        "OK [0]", "User abort [1]", "General MAPI failure [2]", "MAPI login failure [3]",
		        "Disk full [4]", "Insufficient memory [5]", "Access denied [6]", "-unknown- [7]",
		        "Too many sessions [8]", "Too many files were specified [9]", "Too many recipients were specified [10]", 
                "A specified attachment was not found [11]", "Attachment open failure [12]", 
                "Attachment write failure [13]", "Unknown recipient [14]", "Bad recipient type [15]",
		        "No messages [16]", "Invalid message [17]", "Text too large [18]", "Invalid session [19]",
		        "Type not supported [20]", "A recipient was specified ambiguously [21]", "Message in use [22]", 
                "Network failure [23]", "Invalid edit fields [24]", "Invalid recipients [25]", "Not supported [26]" 
		    };

            m_recipients = new List<MapiRecipDesc>();
            m_attachments = new List<string>();
            m_lastError = 0;

        }

        public bool AddRecipient(string email, string name)
        {
            MapiRecipDesc recipient;
            
            recipient = new MapiRecipDesc();
            recipient.recipClass = (int) HowTo.MAPI_TO;
            recipient.address = email;
            recipient.name = name;

            m_recipients.Add(recipient);
            return true;
        }

        public bool AddRecipient(MapiRecipDesc recipient)
        {
            m_recipients.Add(recipient);
            return true;
        }

        public void AddAttachment(string strAttachmentFileName)
        {
            m_attachments.Add(strAttachmentFileName);
        }

        [DllImport("MAPI32.DLL")]
        static extern int MAPISendMail(IntPtr sess, IntPtr hwnd, MapiMessage message, int flg, int rsv);
        [DllImport("MAPI32.DLL")]
        static extern int MAPIResolveName(IntPtr sess, IntPtr hwnd, string name, int flg, int rsv, ref MapiRecipDesc recipient);

        public bool SendMail(string strSubject, string strBody)
        {
            MapiMessage msg;
            
            msg = new MapiMessage();
            msg.subject = strSubject;
            msg.noteText = strBody;
            msg.recips = GetRecipients(out msg.recipCount);
            msg.files = GetAttachments(out msg.fileCount);

            m_lastError = MAPISendMail(new IntPtr(0), new IntPtr(0), msg, MAPI_LOGON_UI | MAPI_DIALOG, 0);
            if (m_lastError > 1)
            {
                MessageBox.Show("MAPISendMail failed! " + GetLastError(), "MAPISendMail", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                Cleanup(ref msg);
                return false;
            }

            Cleanup(ref msg);
            return true;
        }

        public bool ResolveName(string name, ref MapiRecipDesc recipient)
        {

            m_lastError = MAPIResolveName(new IntPtr(0), new IntPtr(0), name, 0, 0, ref recipient);
            if (m_lastError > 1)
            {
                MessageBox.Show("MAPIResolveName failed! " + GetLastError(), "ResolveName", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            return true;
        }


        IntPtr GetRecipients(out int recipCount)
        {
            int size;
            IntPtr intPtr;
            int ptr;

            if (m_recipients.Count == 0)
            {
                recipCount = 0;
                return IntPtr.Zero;
            }

            size = Marshal.SizeOf(typeof(MapiRecipDesc));
            intPtr = Marshal.AllocHGlobal(m_recipients.Count * size);

            ptr = (int)intPtr;

            foreach (MapiRecipDesc mapiDesc in m_recipients)
            {
                Marshal.StructureToPtr(mapiDesc, (IntPtr)ptr, false);
                ptr += size;
            }

            recipCount = m_recipients.Count;
            return intPtr;
        }

        IntPtr GetAttachments(out int fileCount)
        {
            MapiFileDesc mapiFileDesc;
            int size;
            IntPtr intPtr;            
            int ptr;

            if (m_attachments == null)
            {
                fileCount = 0;
                return IntPtr.Zero;
            }

            if ((m_attachments.Count <= 0) || (m_attachments.Count > maxAttachments))
            {
                fileCount = 0;
                return IntPtr.Zero;
            }

            size = Marshal.SizeOf(typeof(MapiFileDesc));
            intPtr = Marshal.AllocHGlobal(m_attachments.Count * size);

            mapiFileDesc = new MapiFileDesc();
            mapiFileDesc.position = -1;
            ptr = (int)intPtr;
            
            foreach (string strAttachment in m_attachments)
            {
                mapiFileDesc.name = Path.GetFileName(strAttachment);
                mapiFileDesc.path = strAttachment;
                Marshal.StructureToPtr(mapiFileDesc, (IntPtr)ptr, false);
                ptr += size;
            }

            fileCount = m_attachments.Count;
            return intPtr;
        }

        void Cleanup(ref MapiMessage msg)
        {
            int size;
            int ptr;
            int i;
            
            if (msg.recips != IntPtr.Zero)
            {
                size = Marshal.SizeOf(typeof(MapiRecipDesc));
                ptr = (int)msg.recips;

                for (i = 0; i < msg.recipCount; i++)
                {
                    Marshal.DestroyStructure((IntPtr)ptr, typeof(MapiRecipDesc));
                    ptr += size;
                }
                Marshal.FreeHGlobal(msg.recips);
            }

            if (msg.files != IntPtr.Zero)
            {
                size = Marshal.SizeOf(typeof(MapiFileDesc));
                ptr = (int)msg.files;

                for (i = 0; i < msg.fileCount; i++)
                {
                    Marshal.DestroyStructure((IntPtr)ptr, typeof(MapiFileDesc));
                    ptr += size;
                }
                Marshal.FreeHGlobal(msg.files);
            }
            
            m_recipients.Clear();
            m_attachments.Clear();
            m_lastError = 0;
        }
        
        public string GetLastError()
		{
		    if (m_lastError <= 26)
			    return errors[ m_lastError ];
		    return "MAPI error [" + m_lastError.ToString() + "]";
		}


    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public class MapiMessage
    {
        public int reserved;
        public string subject;
        public string noteText;
        public string messageType;
        public string dateReceived;
        public string conversationID;
        public int flags;
        public IntPtr originator;
        public int recipCount;
        public IntPtr recips;
        public int fileCount;
        public IntPtr files;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public class MapiFileDesc
    {
        public int reserved;
        public int flags;
        public int position;
        public string path;
        public string name;
        public IntPtr type;
    }

    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
    public class MapiRecipDesc
	{
	    public int		reserved;
	    public int		recipClass;
	    public string	name;
	    public string	address;
	    public int		eIDSize;
	    public IntPtr	entryID;
	}
}
